# Generated by Django 4.1.9 on 2023-06-23 21:02

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("netbox_acls", "0003_netbox_acls"),
    ]

    operations = [
        migrations.AlterField(
            model_name="accesslist",
            name="name",
            field=models.CharField(
                max_length=500,
                validators=[
                    django.core.validators.RegexValidator(
                        "^[a-zA-Z0-9-_]+$", "Only alphanumeric, hyphens, and underscores characters are allowed."
                    )
                ],
            ),
        ),
    ]
