# Generated by Django 4.1.5 on 2023-01-25 20:08

import utilities.json
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("netbox_acls", "0002_alter_accesslist_options_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="accesslist",
            name="custom_field_data",
            field=models.JSONField(
                blank=True,
                default=dict,
                encoder=utilities.json.CustomFieldJSONEncoder,
            ),
        ),
        migrations.AlterField(
            model_name="aclextendedrule",
            name="custom_field_data",
            field=models.JSONField(
                blank=True,
                default=dict,
                encoder=utilities.json.CustomFieldJSONEncoder,
            ),
        ),
        migrations.AlterField(
            model_name="aclinterfaceassignment",
            name="custom_field_data",
            field=models.JSONField(
                blank=True,
                default=dict,
                encoder=utilities.json.CustomFieldJSONEncoder,
            ),
        ),
        migrations.AlterField(
            model_name="aclstandardrule",
            name="custom_field_data",
            field=models.JSONField(
                blank=True,
                default=dict,
                encoder=utilities.json.CustomFieldJSONEncoder,
            ),
        ),
    ]
